@@ -1,5 +1,7 @@
 """Smart energy channels module for Zigbee Home Automation."""
-from typing import Coroutine, Union
+from __future__ import annotations
+
+from typing import Coroutine
 
 import zigpy.zcl.clusters.smartenergy as smartenergy
 
@@ -139,7 +141,7 @@ class Metering(ZigbeeChannel):
         else:
             self._format_spec = "{:0" + str(width) + "." + str(r_digits) + "f}"
 
-    def formatter_function(self, value: int) -> Union[int, float]:
+    def formatter_function(self, value: int) -> int | float:
         """Return formatted value for display."""
         value = value * self.multiplier / self.divisor
         if self.unit_of_measurement == POWER_WATT:
